/* File : APSE.i */
%module Apse

%{
#include "apse.h"
%}

%typemap(out) int * slice{
	int i, size = 3;
	PyObject* obj;
	$result = PyTuple_New(size);
	for (i=0; i<size; i++){
		obj = PyInt_FromLong($1[i]);
		PyTuple_SetItem($result, i, obj);
		Py_DECREF(obj);
	}
}

%include apse.h

%extend apse_t {

	apse_t(char* pattern) {
		apse_size_t pattern_size  = (apse_size_t) strlen(pattern);
		apse_size_t edit_distance = ((pattern_size-1)/10) + 1;
		apse_t *apse = apse_create(pattern, pattern_size, edit_distance);
		apse_set_greedy(apse, 1);
		return apse;
	}
	
	int match(char* text){
		return (int) apse_match(self, text, strlen(text));
	}
	
	long index(char* text){
		return (long) apse_index(self, text, strlen(text));
	}
	
	void set_minimal_distance(int minimal){
		apse_set_minimal_distance(self, minimal);
	}
	
	int* slice(char* text){
		int* slice_arr = (int*) malloc ( sizeof(int) * 3 );
		apse_size_t match_begin=1, match_size=1;
		
		apse_set_minimal_distance(self, 1);		
		apse_slice(self, text, strlen(text), &match_begin, &match_size);
		slice_arr[0] = (int) match_begin;
		slice_arr[1] = (int) match_size;
		slice_arr[2] = self->edit_distance;
		apse_set_minimal_distance(self, 0);		
		return slice_arr;
	}
	
	void set_edit_distance(int edit_distance){
		 apse_set_edit_distance(self, edit_distance);
	}

	void set_insertions(int insertions){
		apse_set_insertions(self, insertions);	
	}

	void set_deletions(int deletions){
		apse_set_deletions(self, deletions);	
	}

	void set_substitutions(int substitutions){
		apse_set_substitutions(self, substitutions);	
	}
	
	int get_edit_distance(){
		return (int) self->edit_distance;
	}

	int get_substitutions(){
		return (int) self->edit_substitutions;
	}

	int get_deletions(){
		return (int) self->edit_deletions;
	}

	int get_insertions(){
		return (int) self->edit_insertions;
	}
	
	void set_caseignore(int len, int state){
		apse_set_caseignore_slice(self, 0, len, state);
	}
}

%pythoncode %{
class Approx:
	"""
	Python extension for approximate matching (fuzzy matching)
	
	Python port of the String::Approx module written for perl 
	by Jarkko Hietaniemi.
	
	APSE.Approx lets you match strings approximately. With this you can emulate 
	errors: typing errorrs, speling errors, closely related vocabularies 
	(colour color), genetic mutations (GAG ACT), abbreviations (McScot, MacScot).
	
	The measure of approximateness is the Levenshtein edit distance. It is the total 
	number of edits defined as: 
	
	insertions, word -> world
	deletions, monkey -> money
	substitutions, sun -> fun
	
	required to transform a string to another string. For example, to 
	transform lead into gold, you need three edits: 
	lead -> gead -> goad -> gold
	The edit distance of lead and gold is therefore three.
	
	Typical usage:
	
	from Apse import Approx
	
	# match entries at most one character away 
	apx = Approx("python", edit=1)
	
	# match strings
	apx.match("jython") # will return the word "jython"
	apx.match("jjthon") # will return None
	
	# match lists
	test_words = ["jython", "jjthon"]
	apx.match(test_words) # returns [ "jython" ]
	
	# edit distances
	apx.dist("jython") # will return 1
	apx.dist("jjthon") # will return 2
	
	Note: The definition of the goodness of an approximate match is the 
	number of steps required to bring the string pattern to a form that is 
	entirely contained in the string to which it is being matched. The process 
	is not commutative. The pattern that you instantiate the class with will be 
	matched against the input. For example the word "funky" can be made to 
	match the word "funnybone" with an edit distance of one. However, using 
	"funnybone" as a pattern that will be  matched to "funky" the distance 
	will become five.
	
	Example:
	
	>>> from Apse import Approx
	>>> a = Approx("funky")
	>>> a.dist("funnybone")
	1
	>>> a = Approx("funnybone")
	>>> a.dist("funky")
	5
	"""
	def __init__(self, pattern, **kwargs):
		"""
		Initializes the matching pattern and optional parameters.
		The following keywords may be used to control the matching
		process:
		
		edit = number
		ins = number
		sub = number
		dlt = number
		
		that govern the number of maximal edits, inserions, substractions
		and deletions. The numbers must either be an integer in which case
		it is interpreted as the absolute number of edits, or a fractional
		number less than 1 in which case it is interpreted as a
		percentage of the length of the pattern.
		
		a = Apse.Approx("python", edit=2, ins=1)
		
		will match only inputs that can be reached by at most two edits 
		only one of which may be an insertion.
		
		a = Apse.Approx("python", edit=0.3)
		
		will match only inputs that can be reached by changing 30%
		of the characters in the word python, that is and edit 
		distance of 2
		
		the presence of the ignore_case keyword argument will 
		lead to matches where the case is ignored.
		"""
		self.apse = apse_t(pattern)
		self.args = kwargs
		self.pattern = pattern
		self.init_args()
		
	def _value(sef, pattern, val):
		if val < 1:
			val = val * len(pattern)
		return int(round((val)))
		
	def init_args(self):
		"""
		Initializes the matching parameters.
		"""
		args, pattern = self.args, self.pattern
		for key in [ "edit", "sub", "ins", "dlt" ]:
			if args.has_key(key): args[key] = self._value(pattern, args[key]) 
			
		if args.has_key("edit"): self.apse.set_edit_distance(args["edit"])
		if args.has_key("sub") : self.apse.set_substitutions(args["sub"])
		if args.has_key("ins") : self.apse.set_insertions(args["ins"])
		if args.has_key("dlt") : self.apse.set_deletions(args["dlt"])
		if args.has_key("ignore_case"): 
			self.apse.set_caseignore(len(self.pattern), 1)
		else:
			self.apse.set_caseignore(len(self.pattern), 0)
		
	def match(self, text):
		"""
		Matches an input to the pattern. The input may be a string or
		a list containing strings. The method returns the matching entry(ies) 
		either as a string or a list.
		"""
		if isinstance(text, type([])):
			ret_val = []
			for line in text:
				if (self.apse.match(line)):
					ret_val.append(line)
		else:
			if self.apse.match(text):
				ret_val = text
			else:
				ret_val = None
				
		return ret_val
	
	# the C library has fields that mutate upon minimal match 
	def dist(self, text):
		"""
		Returns the edit distance (the number of deletions, insertions and
		substitutions needed to match the input string to the pattern
		"""
		old_dist = self.apse.get_edit_distance()
		self.apse.set_minimal_distance(1);
		self.apse.match(text)
		distance = self.apse.get_edit_distance()
		self.apse.set_minimal_distance(0);		
		self.init_args() 
		return distance
		
	def slice(self, text):
		"""
		Returns the starting and ending indices of the locations where 
		the input string matches the pattern plus the edit distance needed 
		to perform the match
		"""
		ret_val =  self.apse.slice(text)
		self.init_args() # restore arguments
		return ret_val
	
	def info(self):
		"""
		Returns a tuple containing the argument dictionary and the pattern
		"""
		return self.args, self.pattern
%}


